/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.admin.common.user;

import static org.assertj.core.api.Assertions.*;

import java.util.Collection;
import java.util.List;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.ValueSource;
import org.mockito.Mock;
import org.mockito.MockedStatic;
import org.mockito.Mockito;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.context.SecurityContext;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.User;

class CurrentUserHelperTest {
	@DisplayName("Has role")
	@Nested
	class TestHasRole {
		@Mock
		private final Authentication mockAuthentication = Mockito.mock(Authentication.class);
		@Mock
		private final User mockPrincipal = Mockito.mock(User.class);

		@Test
		void shouldReturnFalseOnMissingAuthentication() {
			try (MockedStatic<CurrentUserHelper> mockUserHelper = Mockito.mockStatic(
					CurrentUserHelper.class,
					Mockito.CALLS_REAL_METHODS)) {
				mockUserHelper.when(CurrentUserHelper::getAuthentication).thenReturn(null);

				boolean hasRole = CurrentUserHelper.hasRole(UserRole.ADMIN_ADMIN);

				assertThat(hasRole).isFalse();
			}
		}

		@Test
		void shouldReturnFalseOnMissingPrincipal() {
			Mockito.when(mockAuthentication.getPrincipal()).thenReturn(null);
			try (MockedStatic<CurrentUserHelper> mockUserHelper = Mockito.mockStatic(
					CurrentUserHelper.class,
					Mockito.CALLS_REAL_METHODS)) {
				mockUserHelper.when(CurrentUserHelper::getAuthentication).thenReturn(mockAuthentication);

				boolean hasRole = CurrentUserHelper.hasRole(UserRole.ADMIN_ADMIN);

				assertThat(hasRole).isFalse();
			}
		}

		@ParameterizedTest
		@ValueSource(booleans = {false, true})
		void shouldReturnValue(boolean containsRoleValue) {
			Mockito.when(mockAuthentication.getPrincipal()).thenReturn(mockPrincipal);
			List<GrantedAuthority> authorities = List.of();
			Mockito.<Collection<? extends GrantedAuthority>>when(mockAuthentication.getAuthorities()).thenReturn(authorities);

			try (MockedStatic<CurrentUserHelper> mockUserHelper = Mockito.mockStatic(
					CurrentUserHelper.class,
					Mockito.CALLS_REAL_METHODS)) {
				mockUserHelper.when(CurrentUserHelper::getAuthentication).thenReturn(mockAuthentication);
				mockUserHelper.when(() -> CurrentUserHelper.containsRole(Mockito.anyList(), Mockito.anyString()))
						.thenReturn(containsRoleValue);

				boolean hasRole = CurrentUserHelper.hasRole(UserRole.ADMIN_ADMIN);

				mockUserHelper.verify(() -> CurrentUserHelper.containsRole(mockAuthentication.getAuthorities(), UserRole.ADMIN_ADMIN));
				assertThat(hasRole).isEqualTo(containsRoleValue);
			}
		}
	}

	@DisplayName("Contains role")
	@Nested
	class TestContainsRole {
		@Test
		void shouldNotContainRoleIfAuthoritiesIsNull() {
			boolean containsRole = CurrentUserHelper.containsRole(null, UserRole.ADMIN_ADMIN);

			assertThat(containsRole).isFalse();
		}

		@Test
		void shouldNotContainRole() {
			List<GrantedAuthority> authorities = List.of(
					new SimpleGrantedAuthority(CurrentUserHelper.ROLE_PREFIX + "OTHER"));

			boolean containsRole = CurrentUserHelper.containsRole(authorities, UserRole.ADMIN_ADMIN);

			assertThat(containsRole).isFalse();
		}

		@Test
		void shouldContainRole() {
			Collection<? extends GrantedAuthority> authorities = List.of(
					new SimpleGrantedAuthority(CurrentUserHelper.ROLE_PREFIX + UserRole.ADMIN_ADMIN));

			boolean containsRole = CurrentUserHelper.containsRole(authorities, UserRole.ADMIN_ADMIN);

			assertThat(containsRole).isTrue();
		}
	}

	@DisplayName("Add Role Prefix If Missing")
	@Nested
	class TestAddRolePrefixIfMissing {

		@Test
		void shouldAddPrefixIfMissing() {
			var roleWithoutPrefix = UserRole.ADMIN_ADMIN;

			var role = CurrentUserHelper.addRolePrefixIfMissing(roleWithoutPrefix);

			assertThat(role).isEqualTo(CurrentUserHelper.ROLE_PREFIX + UserRole.ADMIN_ADMIN);
		}

		@Test
		void shouldReturnRoleIfPrefixAlreadyExists() {
			var roleWithPrefix = CurrentUserHelper.ROLE_PREFIX + UserRole.ADMIN_ADMIN;

			var role = CurrentUserHelper.addRolePrefixIfMissing(roleWithPrefix);

			assertThat(role).isEqualTo(roleWithPrefix);
		}

		@Test
		void shouldReturnNullIfPassingNull() {
			var role = CurrentUserHelper.addRolePrefixIfMissing(null);

			assertThat(role).isNull();
		}
	}

	@DisplayName("Get authentication")
	@Nested
	class TestGetAuthentication {
		@Mock
		private final SecurityContext mockSecurityContext = Mockito.mock(SecurityContext.class);

		@Test
		void shouldThrowIfNoAuthenticatedUser() {
			Mockito.when(mockSecurityContext.getAuthentication()).thenReturn(null);

			try (MockedStatic<SecurityContextHolder> contextHolder = Mockito.mockStatic(SecurityContextHolder.class)) {
				contextHolder.when(SecurityContextHolder::getContext).thenReturn(mockSecurityContext);

				assertThatIllegalStateException()
						.isThrownBy(CurrentUserHelper::getAuthentication)
						.withMessage("No authenticated User found");
			}
		}

		@Test
		void shouldPassAuthentication() {
			Authentication mockAuthentication = Mockito.mock(Authentication.class);
			Mockito.when(mockSecurityContext.getAuthentication()).thenReturn(mockAuthentication);

			try (MockedStatic<SecurityContextHolder> contextHolder = Mockito.mockStatic(SecurityContextHolder.class)) {
				contextHolder.when(SecurityContextHolder::getContext).thenReturn(mockSecurityContext);

				Authentication authentication = CurrentUserHelper.getAuthentication();

				assertThat(authentication).isSameAs(mockAuthentication);
			}
		}
	}
}
