/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.admin.keycloak;

import static org.assertj.core.api.Assertions.*;
import static org.assertj.core.groups.Tuple.tuple;

import java.util.List;
import java.util.Optional;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.keycloak.representations.idm.GroupRepresentation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ContextConfiguration;

import com.thedeanda.lorem.LoremIpsum;

import de.ozgcloud.admin.common.KeycloakInitializer;
import de.ozgcloud.common.test.ITCase;

@ITCase
@ContextConfiguration(initializers = KeycloakInitializer.class)
class KeycloakApiServiceITCase {

	@Autowired
	private KeycloakApiService service;
	@Autowired
	private KeycloakApiProperties properties;

	@Nested
	class TestGetAllGroups {

		@Test
		void shouldReturnAllTopLevelGroups() {
			var groups = service.getAllGroups();

			assertThat(groups).extracting(GroupRepresentation::getName)
					.contains("GroupWithChild", "GroupWithoutOid", "GroupWithoutOidWithChild");
		}

		@Test
		void shouldTopLevelGroupsHaveAttributes() {
			var groups = service.getAllGroups();

			assertThat(groups).extracting(GroupRepresentation::getName, this::getOrganisationsEinheitId)
					.contains(tuple("GroupWithChild", "GroupWithChild-oid"), tuple("GroupWithoutOid", null),
							tuple("GroupWithoutOidWithChild", null));
		}

		@Test
		void shouldGroupWithChildHaveLevel1Children() {
			var groups = service.getAllGroups();
			var groupWithChild = getTopLevelGroup(groups, "GroupWithChild");

			assertThat(groupWithChild.getSubGroups()).hasSize(2).extracting(GroupRepresentation::getName)
					.containsExactlyInAnyOrder("ChildLevel1WithChild", "ChildLevel1WithoutOid");
		}

		@Test
		void shouldLevel1ChildrenOfGroupWithChildHaveAttributes() {
			var groups = service.getAllGroups();
			var groupWithChild = getTopLevelGroup(groups, "GroupWithChild");

			assertThat(groupWithChild.getSubGroups()).extracting(GroupRepresentation::getName, this::getOrganisationsEinheitId)
					.containsExactlyInAnyOrder(tuple("ChildLevel1WithChild", "ChildLevel1WithChild-oid"), tuple("ChildLevel1WithoutOid", null));
		}

		@Test
		void shouldLevel1ChildHaveLevel2Children() {
			var groups = service.getAllGroups();
			var level1Children = getTopLevelGroup(groups, "GroupWithChild").getSubGroups();
			var level1Child = getTopLevelGroup(level1Children, "ChildLevel1WithChild");

			assertThat(level1Child.getSubGroups()).hasSize(1).extracting(GroupRepresentation::getName).contains("ChildLevel2");
		}

		@Test
		void shouldGroupWithoutOidHaveLevel1Children() {
			var groups = service.getAllGroups();
			var groupWithoutOid = getTopLevelGroup(groups, "GroupWithoutOidWithChild");

			assertThat(groupWithoutOid.getSubGroups()).hasSize(1).extracting(GroupRepresentation::getName)
					.containsExactlyInAnyOrder("ChildLevel1");
		}

		@Test
		void shouldLevel1ChildrenOfGroupWithoutOidHaveAttributes() {
			var groups = service.getAllGroups();
			var groupWithoutOid = getTopLevelGroup(groups, "GroupWithoutOidWithChild");

			assertThat(groupWithoutOid.getSubGroups()).extracting(GroupRepresentation::getName, this::getOrganisationsEinheitId)
					.containsExactlyInAnyOrder(tuple("ChildLevel1", "ChildLevel1-oid"));
		}

		private GroupRepresentation getTopLevelGroup(List<GroupRepresentation> groups, String groupName) {
			var foundGroup = groups.stream().filter(group -> group.getName().equals(groupName)).findFirst();
			assertThat(foundGroup).isPresent();
			return foundGroup.get();
		}

		private String getOrganisationsEinheitId(GroupRepresentation group) {
			var attributes = group.getAttributes();
			return attributes.containsKey(properties.getOrganisationsEinheitIdKey()) ?
					attributes.get(properties.getOrganisationsEinheitIdKey()).getFirst() :
					null;
		}
	}

	@Nested
	class TestAddGroup {

		@Test
		void shouldReturnId() {
			var groupToAdd = createUniqueGroupRepresentation("shouldReturnId");

			var groupId = service.addGroup(groupToAdd);

			assertThat(groupId).isNotBlank();
		}

		@Test
		void shouldAddGroup() {
			var groupToAdd = createUniqueGroupRepresentation("shouldAddGroup");

			var groupId = service.addGroup(groupToAdd);

			assertThat(findGroupInKeycloak(groupId)).isPresent().get().extracting(GroupRepresentation::getName, GroupRepresentation::getAttributes)
					.containsExactly(groupToAdd.getName(), groupToAdd.getAttributes());
		}

		@Test
		void shouldThrowExceptionWhenAddingGroupWithId() {
			var groupToAdd = GroupRepresentationTestFactory.create();

			assertThatExceptionOfType(ResourceCreationException.class).isThrownBy(() -> service.addGroup(groupToAdd))
					.withMessageContaining("404");
		}

		@Test
		void shouldThrowExceptionWhenNameIsNotUnique() {
			var groupToAdd = createUniqueGroupRepresentation("notUniqueName");

			service.addGroup(groupToAdd);

			assertThatExceptionOfType(ResourceCreationException.class).isThrownBy(() -> service.addGroup(groupToAdd))
					.withMessageContaining("409");
		}

		@Test
		void shouldNotAddSubgroupsOfAddedGroup() {
			var groupToAdd = createUniqueGroupRepresentation("shouldNotAddSubgroups");

			var groupId = service.addGroup(groupToAdd);

			assertThat(findGroupInKeycloak(groupId)).isPresent().get().extracting(GroupRepresentation::getSubGroups)
					.asList().isEmpty();
		}

		private GroupRepresentation createUniqueGroupRepresentation(String nameSuffix) {
			// LoremIpsum does not guarantee unique results when called repeatedly
			var groupName = "%s (%s)".formatted(LoremIpsum.getInstance().getName(), nameSuffix);
			var group = GroupRepresentationTestFactory.createWithoutId(groupName);
			group.setParentId(null);
			return group;
		}

		private Optional<GroupRepresentation> findGroupInKeycloak(String groupId) {
			return service.getAllGroups().stream()
					.filter(groupRepresentation -> groupId.equals(groupRepresentation.getId()))
					.findFirst();
		}
	}
}
